@@ -40,20 +40,14 @@ static GalagoService *gossip_galago_get_service          (GossipAccount  *accoun
 static GalagoAccount *galago_get_account                 (GossipAccount  *account);
 static void           galago_set_status                  (GalagoAccount  *account,
 							  GossipPresence *presence);
-static void           galago_presence_changed_cb         (GossipSession  *gossip_session,
-							  GossipPresence *gossip_presence,
-							  gpointer        userdata);
 static void           galago_contact_added_cb            (GossipSession  *session,
 							  GossipContact  *contact,
 							  gpointer        user_data);
 static void           galago_contact_removed_cb          (GossipSession  *session,
 							  GossipContact  *contact,
 							  gpointer        user_data);
-static void           galago_contact_updated_cb          (GossipSession  *session,
-							  GossipContact  *contact,
-							  gpointer        user_data);
-static void           galago_contact_presence_updated_cb (GossipSession  *session,
-							  GossipContact  *contact,
+static void           galago_contact_presence_updated_cb (GossipContact  *contact,
+							  GParamSpec     *param,
 							  gpointer        user_data);
 static void           galago_setup_accounts              (GossipSession  *session);
 
@@ -192,37 +186,6 @@ galago_set_status (GalagoAccount  *account,
 	galago_presence_add_status (gp, galago_status_new (gst, id, status, TRUE));
 }
 
-static void
-galago_presence_changed_cb (GossipSession  *session,
-			    GossipPresence *presence,
-			    gpointer        userdata)
-{
-	GList *accounts;
-	GList *l;
-
-	gossip_debug (DEBUG_DOMAIN, "Session presence changed");
-
-	accounts = gossip_session_get_accounts (session);
-
-	for (l = accounts; l != NULL; l = l->next) {
-		GossipAccount *account;
-		GalagoAccount *ga;
-
-		account = GOSSIP_ACCOUNT (l->data);
-
-		/* Only set status to presence for connected accounts */
-		if (!gossip_session_is_connected (session, account)) {
-			continue;
-		}
-
-		ga = galago_get_account (account);
-		galago_set_status (ga, presence);
-	}
-
-	g_list_foreach (accounts, (GFunc) g_object_unref, NULL);
-	g_list_free (accounts);
-}
-
 static void
 galago_contact_added_cb (GossipSession *session,
 			 GossipContact *contact,
@@ -239,6 +202,10 @@ galago_contact_added_cb (GossipSession *session,
 
 	gossip_debug (DEBUG_DOMAIN, "Added contact:'%s'", contact_id);
 
+	g_signal_connect (contact, "notify::presences",
+			  G_CALLBACK (galago_contact_presence_updated_cb),
+			  NULL);
+
 	ga_me = galago_get_account (account);
 	gs = galago_account_get_service (ga_me);
 
@@ -277,6 +244,10 @@ galago_contact_removed_cb (GossipSession *session,
 
 	gossip_debug (DEBUG_DOMAIN, "Contact removed:'%s'", contact_id);
 
+	g_signal_handlers_disconnect_by_func (contact,
+					      galago_contact_presence_updated_cb,
+					      NULL);
+
 	ga_me = galago_get_account (account);
 	gs = galago_account_get_service (ga_me);
 
@@ -302,19 +273,8 @@ galago_contact_removed_cb (GossipSession *session,
 }
 
 static void
-galago_contact_updated_cb (GossipSession  *session,
-			   GossipContact  *contact,
-			   gpointer        user_data)
-{
-	gossip_debug (DEBUG_DOMAIN, "Contact updated:'%s'",
-		      gossip_contact_get_id (contact));
-
-	/* TODO */
-}
-
-static void
-galago_contact_presence_updated_cb (GossipSession *session,
-				    GossipContact *contact,
+galago_contact_presence_updated_cb (GossipContact *contact,
+				    GParamSpec    *param,
 				    gpointer       user_data)
 {
 	GossipAccount  *account;
@@ -375,7 +335,7 @@ galago_setup_accounts (GossipSession *session)
 
 		presence = gossip_session_get_presence (session);
 		if (presence) {
-			galago_presence_changed_cb (session, presence, NULL);
+			galago_set_status (ga, presence);
 		}
 	}
 
@@ -406,22 +366,10 @@ gossip_galago_init (GossipSession *session)
 
 	galago_setup_accounts (session);
 
-	g_signal_connect (session,
-			  "presence-changed",
-			  G_CALLBACK (galago_presence_changed_cb),
-			  NULL);
 	g_signal_connect (session,
 			  "contact-added",
 			  G_CALLBACK (galago_contact_added_cb),
 			  NULL);
-	g_signal_connect (session,
-			  "contact-updated",
-			  G_CALLBACK (galago_contact_updated_cb),
-			  NULL);
-	g_signal_connect (session,
-			  "contact-presence-updated",
-			  G_CALLBACK (galago_contact_presence_updated_cb),
-			  NULL);
 	g_signal_connect (session,
 			  "contact-removed",
 			  G_CALLBACK (galago_contact_removed_cb),
