@@ -21,6 +21,12 @@
 #ifndef __GOSSIP_VCARD_DIALOG_H__
 #define __GOSSIP_VCARD_DIALOG_H__
 
+#include <gtk/gtkwindow.h>
+
+G_BEGIN_DECLS
+
 void gossip_vcard_dialog_show (GtkWindow *parent);
 
+G_END_DECLS
+
 #endif /* __GOSSIP_VCARD_DIALOG_H__ */
